#ifndef EVENTS_H
#define EVENTS_H

#include <qdialog.h>
#include <qstring.h>
#include <qdatetime.h>
#include <qtimer.h>

#include "libgadu.h"
#include "misc.h"

class AutoConnectionTimer : private QTimer {
	Q_OBJECT

	public:
		static void on();
		static void off();

	public slots:
		void doConnect();
	
	private:
		AutoConnectionTimer(QObject *parent = 0);

		static AutoConnectionTimer *autoconnection_object;
};

class SavePublicKey : public QDialog {
	Q_OBJECT
	public:
		SavePublicKey(uin_t uin, QString keyData, QWidget *parent = 0, const char *name = 0);

	private:
		uin_t uin;
		QString keyData;

	private slots:
		void yesClicked();
};

/**
	Menader zdarze. Udostpnia sygnay reprezentujce zdarzenia
	otrzymywane z serwera GG.
**/
class EventManager : public QObject
{
	Q_OBJECT

	private slots:
		void connectedSlot();
		void connectionFailedSlot();
		void connectionBrokenSlot();
		void disconnectedSlot();
		void userStatusChangedSlot(struct gg_event*);
		void userlistReceivedSlot(struct gg_event *);
		void messageReceivedSlot(int, UinsList,unsigned char *msg,time_t,
			int formats_length, void *formats);
		void chatReceivedSlot(UinsList senders,const QString& msg,time_t time);
		void ackReceivedSlot(int seq);
		void dccConnectionReceivedSlot(const UserListElement& sender);
		void pubdirReplyReceivedSlot(gg_pubdir50_t res);
		
	public:
		EventManager();
		void eventHandler(gg_session* sess);

	signals:
		/**
			Nawizano poczenie z serwerem
		**/
		void connected();
		/**
			Bd poczenia z serwerem
		**/
		void connectionFailed();
		/**
			Poczenia z serwerem zostao przerwane
		**/
		void connectionBroken();
		/**
			Otrzymano polecenie rozczenia
		**/
		void disconnected();		
		/**
			Ktry z kontaktw zmieni swj status
		**/
		void userStatusChanged(struct gg_event*);
		/**
			Przysza z serwera lista kontaktw
		**/
		void userlistReceived(struct gg_event *);
		/**
			Otrzymano jak wiadomo z serwera GG
		**/
		void messageReceived(int,UinsList,unsigned char* msg,time_t,
			int formats_length, void *formats);
		/**
			Otrzymano wiadomo, ktr trzeba pokaza (klasa chat lub msg,
			nadawca nie jest ignorowany, itp)
			Tre zdeszyfrowana i zdekodowana do unicode.
		**/
		void chatReceived(UinsList senders,const QString& msg,time_t time);
		/**
			Otrzymano potwierdzenie wiadomoci
		**/
		void ackReceived(int seq);
		/**
			Otrzymano DCC CTCP
		**/
		void dccConnectionReceived(const UserListElement& sender);
		/**
			Otrzymano z serwera rezultat operacji wyszukiwania
			w katalogu publicznym
		**/		
		void pubdirReplyReceived(gg_pubdir50_t res);
};

extern EventManager event_manager;

extern QTime lastsoundtime;

#endif
